@@ -121,7 +121,7 @@ class GXDLMS:
         if (reply.moreData & RequestTypes.FRAME) != 0:
             id_ = settings.getReceiverReady()
             return GXDLMS.getHdlcFrame(settings, id_, None)
-        if settings.useLogicalNameReferencing:
+        if settings.getUseLogicalNameReferencing():
             if settings.isServer:
                 cmd = Command.GET_RESPONSE
             else:
@@ -142,12 +142,12 @@ class GXDLMS:
         else:
             #  Get next block.
             bb = GXByteBuffer(4)
-            if settings.useLogicalNameReferencing:
+            if settings.getUseLogicalNameReferencing():
                 bb.setUInt32(settings.blockIndex)
             else:
                 bb.setUInt16(settings.blockIndex)
             settings.increaseBlockIndex()
-            if settings.useLogicalNameReferencing:
+            if settings.getUseLogicalNameReferencing():
                 p = GXDLMSLNParameters(settings, 0, cmd, GetCommandType.NEXT_DATA_BLOCK, bb, None, 0xff)
                 reply = GXDLMS.getLnMessages(p)
             else:
@@ -481,7 +481,7 @@ class GXDLMS:
         key = None
         cipher = p.settings.cipher
         if (p.settings.negotiatedConformance & Conformance.GENERAL_PROTECTION) == Conformance.NONE:
-            if (p.settings.connected & ConnectionState.DLMS) != 0 and cipher.dedicatedKey:
+            if cipher.dedicatedKey and (not p.settings.isServer or (p.settings.connected & ConnectionState.DLMS) != 0):
                 cmd = cls.getDedMessage(p.command)
                 key = cipher.dedicatedKey
             else:
@@ -793,6 +793,8 @@ class GXDLMS:
         crcRead = 0
         if reply.size - reply.position < 9:
             data.complete = False
+            if notify:
+                notify.complete = False
             return 0
         data.complete = True
         if notify:
@@ -806,6 +808,8 @@ class GXDLMS:
             pos += 1
         if reply.position == len(reply):
             data.complete = False
+            if notify:
+                notify.complete = False
             return 0
         frame_ = reply.getUInt8()
         if (frame_ & 0xF0) != 0xA0:
@@ -817,6 +821,8 @@ class GXDLMS:
         frameLen += ch
         if len(reply) - reply.position + 1 < frameLen:
             data.complete = False
+            if notify:
+                notify.complete = False
             reply.position = packetStartID
             return 0
         eopPos = frameLen + packetStartID + 1
@@ -838,6 +844,10 @@ class GXDLMS:
                 isNotify = True
                 notify.clientAddress = addresses[1]
                 notify.serverAddress = addresses[0]
+        # HDLC control fields
+        cf = reply.getUInt8()
+        if notify and cf == 0x13:
+            isNotify = True
         if (frame_ & 0x8) != 0:
             if isNotify:
                 notify.moreData = notify.moreData | RequestTypes.FRAME
@@ -848,8 +858,7 @@ class GXDLMS:
                 notify.moreData = notify.moreData & ~RequestTypes.FRAME
             else:
                 data.moreData = data.moreData & ~RequestTypes.FRAME
-        frame_ = reply.getUInt8()
-        if data.xml is None and not settings.checkFrame(frame_):
+        if data.xml is None and not settings.checkFrame(cf):
             reply.position = eopPos + 1
             return GXDLMS.getHdlcData(server, settings, reply, data, notify)
         crc = _GXFCS16.countFCS16(reply, packetStartID + 1, reply.position - packetStartID - 1)
@@ -872,16 +881,16 @@ class GXDLMS:
                 notify.packetLength = (reply.position + 1)
             else:
                 data.packetLength = (reply.position + 1)
-        if frame_ != 0x13 and (frame_ & HdlcFrameType.U_FRAME) == HdlcFrameType.U_FRAME:
+        if cf != 0x13 and (cf & HdlcFrameType.U_FRAME) == HdlcFrameType.U_FRAME:
             if reply.position == packetStartID + frameLen + 1:
                 reply.getUInt8()
-            if frame_ == 0x97:
+            if cf == 0x97:
                 data.error = ErrorCode.UNACCEPTABLE_FRAME
-            elif frame_ == 0x1f:
+            elif cf == 0x1f:
                 data.error = ErrorCode.DISCONNECT_MODE
-            data.command = (frame_)
-        elif frame_ != 0x13 and (frame_ & HdlcFrameType.S_FRAME) == HdlcFrameType.S_FRAME:
-            tmp = (frame_ >> 2) & 0x3
+            data.command = cf
+        elif cf != 0x13 and (cf & HdlcFrameType.S_FRAME) == HdlcFrameType.S_FRAME:
+            tmp = (cf >> 2) & 0x3
             if tmp == HdlcControlFrame.REJECT:
                 data.error = ErrorCode.REJECTED
             elif tmp == HdlcControlFrame.RECEIVE_NOT_READY:
@@ -893,12 +902,12 @@ class GXDLMS:
         else:
             if reply.position == packetStartID + frameLen + 1:
                 reply.getUInt8()
-                if (frame_ & 0x1) == 0x1:
+                if (cf & 0x1) == 0x1:
                     data.moreData = (RequestTypes.FRAME)
             else:
                 if not cls.getLLCBytes(server, reply) and data.xml:
                     cls.getLLCBytes(not server, reply)
-        return frame_
+        return cf
 
     @classmethod
     def getServerAddress(cls, address, logical, physical):
@@ -1109,8 +1118,8 @@ class GXDLMS:
             if data.available() == 0:
                 if cnt != 1:
                     cls.getDataFromBlock(reply.data, 0)
-                    reply.setValue(values)
-                    reply.setReadPosition(data.position)
+                    reply.value = values
+                    reply.readPosition = data.position
                 return False
             if first:
                 type_ = data.getUInt8()
@@ -1417,21 +1426,48 @@ class GXDLMS:
         if data.xml:
             data.xml.appendEndTag(Command.WRITE_RESPONSE)
 
+    @classmethod
+    def handleGetResponseWithList(cls, settings, reply):
+        cnt = _GXCommon.getObjectCount(reply.data)
+        values = list([None] * cnt)
+        if reply.xml:
+            reply.xml.appendStartTag(TranslatorTags.RESULT, "Qty", reply.xml.integerToHex(cnt, 2))
+        pos = 0
+        while pos != cnt:
+            ch = reply.data.getUInt8()
+            if ch != 0:
+                reply.error = reply.data.getUInt8()
+            else:
+                if reply.xml:
+                    di = _GXDataInfo()
+                    di.xml = (reply.xml)
+                    reply.xml.appendStartTag(Command.READ_RESPONSE, SingleReadResponse.DATA)
+                    _GXCommon.getData(reply.data, di)
+                    reply.xml.appendEndTag(Command.READ_RESPONSE, SingleReadResponse.DATA)
+                else:
+                    reply.readPosition = reply.data.position
+                    cls.getValueFromData(settings, reply)
+                    reply.data.position = reply.readPosition
+                    if values:
+                        values[pos] = reply.value
+                    reply.value = None
+            pos += 1
+        reply.value = values
+
     @classmethod
     def handleGetResponse(cls, settings, reply, index):
         # pylint: disable=too-many-locals
         ret = True
-        data = reply.data
-        type_ = data.getUInt8()
-        ch = data.getUInt8()
+        type_ = reply.data.getUInt8()
+        ch = reply.data.getUInt8()
         if reply.xml:
             reply.xml.appendStartTag(Command.GET_RESPONSE)
             reply.xml.appendStartTag(Command.GET_RESPONSE, type_)
             reply.xml.appendLine(TranslatorTags.INVOKE_ID, "Value", reply.xml.integerToHex(ch, 2))
         if type_ == GetCommandType.NORMAL:
-            ch = data.getUInt8()
+            ch = reply.data.getUInt8()
             if ch != 0:
-                reply.error = data.getUInt8()
+                reply.error = reply.data.getUInt8()
             if reply.xml:
                 reply.xml.appendStartTag(TranslatorTags.RESULT)
                 if reply.getError() != 0:
@@ -1443,9 +1479,9 @@ class GXDLMS:
                     _GXCommon.getData(reply.data, di)
                     reply.xml.appendEndTag(TranslatorTags.DATA)
             else:
-                cls.getDataFromBlock(data, 0)
+                cls.getDataFromBlock(reply.data, 0)
         elif type_ == GetCommandType.NEXT_DATA_BLOCK:
-            ch = data.getUInt8()
+            ch = reply.data.getUInt8()
             if reply.xml:
                 reply.xml.appendStartTag(TranslatorTags.RESULT)
                 reply.xml.appendLine(TranslatorTags.LAST_BLOCK, "Value", reply.xml.integerToHex(ch, 2))
@@ -1453,7 +1489,7 @@ class GXDLMS:
                 reply.moreData = reply.moreData | RequestTypes.DATABLOCK
             else:
                 reply.moreData = reply.moreData & ~RequestTypes.DATABLOCK
-            number = data.getUInt32()
+            number = reply.data.getUInt32()
             if reply.xml:
                 reply.xml.appendLine(TranslatorTags.BLOCK_NUMBER, "Value", reply.xml.integerToHex(number, 8))
             else:
@@ -1462,60 +1498,39 @@ class GXDLMS:
                 expectedIndex = settings.blockIndex
                 if number != expectedIndex:
                     raise ValueError("Invalid Block number. It is " + number + " and it should be " + expectedIndex + ".")
-            ch = data.getUInt8()
+            ch = reply.data.getUInt8()
             if ch != 0:
-                reply.error = data.getUInt8()
+                reply.error = reply.data.getUInt8()
             if reply.xml:
                 reply.xml.appendStartTag(TranslatorTags.RESULT)
                 if reply.getError() != 0:
                     reply.xml.appendLine(TranslatorTags.DATA_ACCESS_RESULT, "Value", GXDLMS.errorCodeToString(reply.xml.outputType, reply.error))
-                elif data.available() != 0:
-                    blockLength = _GXCommon.getObjectCount(data)
+                elif reply.data.available() != 0:
+                    blockLength = _GXCommon.getObjectCount(reply.data)
                     if (reply.moreData & RequestTypes.FRAME) == 0:
-                        if blockLength > len(data) - data.position:
-                            reply.xml.appendComment("Block is not complete." + str(len(data) - data.position) + "/" + str(blockLength) + ".")
-                    reply.xml.appendLine(TranslatorTags.RAW_DATA, "Value", data.toHex(False, data.position, data.available()))
+                        if blockLength > len(reply.data) - reply.data.position:
+                            reply.xml.appendComment("Block is not complete." + str(len(reply.data) - reply.data.position) + "/" + str(blockLength) + ".")
+                    reply.xml.appendLine(TranslatorTags.RAW_DATA, "Value", reply.data.toHex(False, reply.data.position, reply.data.available()))
                 reply.xml.appendEndTag(TranslatorTags.RESULT)
-            elif data.position != len(data):
-                blockLength = _GXCommon.getObjectCount(data)
+            elif reply.data.position != len(reply.data):
+                blockLength = _GXCommon.getObjectCount(reply.data)
                 if (reply.moreData & RequestTypes.FRAME) == 0:
-                    if blockLength > len(data) - data.position:
+                    if blockLength > len(reply.data) - reply.data.position:
                         raise ValueError("Invalid block length.")
-                    reply.command = (Command.NONE)
+                    reply.command = Command.NONE
                 if blockLength == 0:
-                    data.size = index
+                    reply.data.size = index
                 else:
-                    cls.getDataFromBlock(data, index)
+                    cls.getDataFromBlock(reply.data, index)
                 if reply.moreData == RequestTypes.NONE:
                     if not reply.peek:
-                        data.position = 0
+                        reply.data.position = 0
                         settings.resetBlockIndex()
+                if reply.moreData == RequestTypes.NONE and settings and settings.command == Command.GET_REQUEST and settings.commandType == GetCommandType.WITH_LIST:
+                    cls.handleGetResponseWithList(settings, reply)
+                    ret = False
         elif type_ == GetCommandType.WITH_LIST:
-            cnt = _GXCommon.getObjectCount(data)
-            values = list([None] * cnt)
-            if reply.xml:
-                reply.xml.appendStartTag(TranslatorTags.RESULT, "Qty", reply.xml.integerToHex(cnt, 2))
-            pos = 0
-            while pos != cnt:
-                ch = data.getUInt8()
-                if ch != 0:
-                    reply.error = data.getUInt8()
-                else:
-                    if reply.xml:
-                        di = _GXDataInfo()
-                        di.xml = (reply.xml)
-                        reply.xml.appendStartTag(Command.READ_RESPONSE, SingleReadResponse.DATA)
-                        _GXCommon.getData(reply.data, di)
-                        reply.xml.appendEndTag(Command.READ_RESPONSE, SingleReadResponse.DATA)
-                    else:
-                        reply.readPosition = data.position
-                        cls.getValueFromData(settings, reply)
-                        data.position = reply.readPosition
-                        if values:
-                            values[pos] = reply.value
-                        reply.value = None
-                pos += 1
-            reply.value = values
+            cls.handleGetResponseWithList(settings, reply)
             ret = False
         else:
             raise ValueError("Invalid Get response.")
@@ -1621,7 +1636,7 @@ class GXDLMS:
                 Command.GLO_METHOD_REQUEST, Command.DED_GET_REQUEST, Command.DED_SET_REQUEST, Command.DED_METHOD_REQUEST):
                 cls.handleGloDedRequest(settings, data)
             elif cmd in (Command.GLO_READ_RESPONSE, Command.GLO_WRITE_RESPONSE, Command.GLO_GET_RESPONSE, Command.GLO_SET_RESPONSE, \
-                Command.GLO_METHOD_RESPONSE, Command.GENERAL_GLO_CIPHERING, Command.GLO_EVENT_NOTIFICATION_REQUEST, \
+                Command.GLO_METHOD_RESPONSE, Command.GENERAL_GLO_CIPHERING, Command.GLO_EVENT_NOTIFICATION, \
                 Command.DED_GET_RESPONSE, Command.DED_SET_RESPONSE, Command.DED_METHOD_RESPONSE, Command.GENERAL_DED_CIPHERING, Command.DED_EVENT_NOTIFICATION):
                 cls.handleGloDedResponse(settings, data, index)
             elif cmd == Command.DATA_NOTIFICATION:
@@ -1657,7 +1672,7 @@ class GXDLMS:
                 data.command = Command.NONE
             elif settings.isServer:
                 if cmd in (Command.GLO_READ_REQUEST, Command.GLO_WRITE_REQUEST, Command.GLO_GET_REQUEST, Command.GLO_SET_REQUEST, \
-                    Command.GLO_METHOD_REQUEST, Command.GLO_EVENT_NOTIFICATION_REQUEST, Command.DED_GET_REQUEST, Command.DED_SET_REQUEST, \
+                    Command.GLO_METHOD_REQUEST, Command.GLO_EVENT_NOTIFICATION, Command.DED_GET_REQUEST, Command.DED_SET_REQUEST, \
                     Command.DED_METHOD_REQUEST, Command.DED_EVENT_NOTIFICATION):
                     data.command = (Command.NONE)
                     data.data.position = data.getCipherIndex()
@@ -1735,7 +1750,7 @@ class GXDLMS:
             bb = GXByteBuffer(data.data)
             data.data.size = data.data.position = index
             p = None
-            if settings.cipher.dedicatedKey and settings.connected == ConnectionState.DLMS:
+            if settings.cipher.dedicatedKey and (not settings.isServer or settings.connected == ConnectionState.DLMS):
                 p = AesGcmParameter(0, settings.sourceSystemTitle, settings.cipher.dedicatedKey, settings.cipher.authenticationKey)
             else:
                 p = AesGcmParameter(0, settings.sourceSystemTitle, settings.cipher.blockCipherKey, settings.cipher.authenticationKey)
@@ -1845,6 +1860,23 @@ class GXDLMS:
         if target.xml or ((frame_ != 0x13 or moreData) and (frame_ & 0x1) != 0):
             return True
         cls.getPdu(settings, target)
+        if notify and not isNotify:
+            #Check command to make sure it's not notify message.
+            if data.command in (Command.DATA_NOTIFICATION,
+                                Command.GLO_EVENT_NOTIFICATION,
+                                Command.INFORMATION_REPORT,
+                                Command.EVENT_NOTIFICATION,
+                                Command.DED_INFORMATION_REPORT_REQUEST,
+                                Command.DED_EVENT_NOTIFICATION):
+                isNotify = True
+                notify.complete = data.complete
+                notify.command = data.command
+                data.command = Command.NONE
+                notify.time = data.time
+                data.time = None
+                notify.data.set(data.data)
+                notify.value = data.value
+                data.data.trim()
         if isNotify:
             return False
         return True
