@@ -80,7 +80,7 @@ class GXDLMSClient(object):
     def __init__(self, useLogicalNameReferencing=True, clientAddress=16, serverAddress=1, forAuthentication=Authentication.NONE, password=None, interfaceType=InterfaceType.HDLC):
         # DLMS settings.
         self.settings = GXDLMSSettings(False)
-        self.useLogicalNameReferencing = useLogicalNameReferencing
+        self.settings.setUseLogicalNameReferencing(useLogicalNameReferencing)
         self.clientAddress = clientAddress
         self.serverAddress = serverAddress
         self.authentication = forAuthentication
@@ -203,10 +203,10 @@ class GXDLMSClient(object):
     maxReceivePDUSize = property(__getMaxReceivePDUSize, __setMaxReceivePDUSize)
 
     def __getUseLogicalNameReferencing(self):
-        return self.settings.useLogicalNameReferencing
+        return self.settings.getUseLogicalNameReferencing()
 
     def __setUseLogicalNameReferencing(self, value):
-        self.settings.useLogicalNameReferencing = value
+        self.settings.setUseLogicalNameReferencing(value)
 
     #
     # Determines, whether Logical, or Short name, referencing is used.
@@ -365,6 +365,7 @@ class GXDLMSClient(object):
     # Interface type.
     #
     interfaceType = property(__getInterfaceType, __setInterfaceType)
+    """Interface type."""
 
     #
     # Information from the connection size that server can
@@ -490,7 +491,7 @@ class GXDLMSClient(object):
             self.settings.setCtoSChallenge(None)
         _GXAPDU.generateAarq(self.settings, self.settings.cipher, None, buff)
         reply = None
-        if self.settings.useLogicalNameReferencing:
+        if self.settings.getUseLogicalNameReferencing():
             p = GXDLMSLNParameters(self.settings, 0, Command.AARQ, 0, buff, None, 0xff)
             reply = GXDLMS.getLnMessages(p)
         else:
@@ -918,7 +919,7 @@ class GXDLMSClient(object):
         if self.autoIncreaseInvokeID:
             self.settings.setInvokeID(int(((self.settings.invokeId + 1) & 0xF)))
         if self.useLogicalNameReferencing:
-            attributeDescriptor.setUInt16(objectType)
+            attributeDescriptor.setUInt16(int(objectType))
             attributeDescriptor.set(_GXCommon.logicalNameToBytes(str(name)))
             attributeDescriptor.setUInt8(attributeOrdinal)
             if not data:
@@ -1223,7 +1224,7 @@ class GXDLMSClient(object):
             return reply.value
         raise ValueError("Invalid command. " + reply.command)
 
-    def ParsePushObjects(self, data):
+    def parsePushObjects(self, data):
         objects = list()
         if data:
             c = GXDLMSConverter(self.standard)
@@ -1232,7 +1233,7 @@ class GXDLMSClient(object):
                 classID = tmp[0]
                 if classID > 0:
                     comp = None
-                    comp = self.objects.findByLN(ObjectType(classID), _GXCommon.toLogicalName(int(tmp[1])))
+                    comp = self.objects.findByLN(classID, _GXCommon.toLogicalName(tmp[1]))
                     if comp is None:
                         comp = self.__createDLMSObject(classID, 0, 0, tmp[1], None)
                         self.settings.objects.append(comp)
