@@ -136,10 +136,10 @@ class GXDLMSServer:
     maxReceivePDUSize = property(__getMaxReceivePDUSize, __setMaxReceivePDUSize)
 
     def __getUseLogicalNameReferencing(self):
-        return self.useLogicalNameReferencing
+        return self.settings.getUseLogicalNameReferencing()
 
     def __setUseLogicalNameReferencing(self, value):
-        self.useLogicalNameReferencing = value
+        self.settings.setUseLogicalNameReferencing(value)
 
     #
     # Determines, whether Logical, or Short name, referencing is used.
@@ -393,7 +393,7 @@ class GXDLMSServer:
         if push is None:
             raise ValueError("push")
         buff = GXByteBuffer()
-        buff.setUInt8(int(DataType.STRUCTURE))
+        buff.setUInt8(DataType.STRUCTURE)
         _GXCommon.setObjectCount(len(push.pushObjectList), buff)
         for k, v in push.pushObjectList:
             self.addData(k, v.attributeIndex, buff)
@@ -802,7 +802,7 @@ class GXDLMSServer:
             cmd = Command.METHOD_RESPONSE
         else:
             cmd = Command.NONE
-        if self.settings.useLogicalNameReferencing:
+        if self.settings.getUseLogicalNameReferencing():
             p = GXDLMSLNParameters(self.settings, 0, cmd, 1, None, None, error)
             GXDLMS.getLNPdu(p, self.replyData)
         else:
