@@ -146,7 +146,7 @@ class _GXCommon:
     #
     @classmethod
     def addString(cls, value, bb):
-        bb.setUInt8(int(DataType.OCTET_STRING))
+        bb.setUInt8(DataType.OCTET_STRING)
         if not value:
             _GXCommon.setObjectCount(0, bb)
         else:
@@ -307,9 +307,12 @@ class _GXCommon:
     #
     @classmethod
     def integerToHex(cls, value, desimals):
-        nbits = desimals * 4
-        str_ = hex((value + (1 << nbits)) % (1 << nbits))[2:].upper()
-        if desimals == 0 or desimals == len(str_):
+        if desimals:
+            nbits = desimals * 4
+            str_ = hex((value + (1 << nbits)) % (1 << nbits))[2:].upper()
+        else:
+            str_ = hex(value)[2:].upper()
+        if not desimals or desimals == len(str_):
             return str_
         return _GXCommon.zeroes[0: desimals - len(str_)] + str_.upper()
 
@@ -389,7 +392,7 @@ class _GXCommon:
         value = None
         if len(buff) - buff.position < 4:
             #  If there is not enough data available.
-            info.complete = (False)
+            info.complete = False
             return None
         str_ = None
         if info.xml:
@@ -441,7 +444,7 @@ class _GXCommon:
         value = None
         if len(buff) - buff.position < 5:
             #  If there is not enough data available.
-            info.complete = (False)
+            info.complete = False
             return None
         str_ = None
         if info.xml:
@@ -509,7 +512,7 @@ class _GXCommon:
         extra = DateTimeExtraInfo.NONE
         #  If there is not enough data available.
         if len(buff) - buff.position < 12:
-            info.complete = (False)
+            info.complete = False
             return None
         str_ = None
         if info.xml:
@@ -542,7 +545,7 @@ class _GXCommon:
                 deviation = 0x8000
                 skip |= DateTimeSkips.DEVITATION
             status = buff.getUInt8()
-            dt.status = ClockStatus(status)
+            dt.status = status
             if year < 1900 or year == 0xFFFF:
                 skip |= DateTimeSkips.YEAR
                 year = 2000
@@ -614,7 +617,7 @@ class _GXCommon:
         value = None
         #  If there is not enough data available.
         if len(buff) - buff.position < 8:
-            info.complete = (False)
+            info.complete = False
             return None
         value = buff.getDouble()
         if info.xml:
@@ -639,7 +642,7 @@ class _GXCommon:
         value = None
         #  If there is not enough data available.
         if len(buff) - buff.position < 4:
-            info.complete = (False)
+            info.complete = False
             return None
         value = buff.getFloat()
         if info.xml:
@@ -664,7 +667,7 @@ class _GXCommon:
         value = None
         #  If there is not enough data available.
         if len(buff) - buff.position < 1:
-            info.complete = (False)
+            info.complete = False
             return None
         value = buff.getUInt8()
         if info.xml:
@@ -685,7 +688,7 @@ class _GXCommon:
         value = None
         #  If there is not enough data available.
         if len(buff) - buff.position < 8:
-            info.complete = (False)
+            info.complete = False
             return None
         value = buff.getUInt64()
         if info.xml:
@@ -706,7 +709,7 @@ class _GXCommon:
         value = None
         #  If there is not enough data available.
         if len(buff) - buff.position < 8:
-            info.complete = (False)
+            info.complete = False
             return None
         value = buff.getInt64()
         if info.xml:
@@ -727,7 +730,7 @@ class _GXCommon:
         value = None
         #  If there is not enough data available.
         if len(buff) - buff.position < 2:
-            info.complete = (False)
+            info.complete = False
             return None
         value = buff.getUInt16()
         if info.xml:
@@ -824,7 +827,7 @@ class _GXCommon:
 
         #  If there is not enough data available.
         if len(buff) - buff.position < 2:
-            info.complete = (False)
+            info.complete = False
             return None
         dt = buff.getUInt8()
         if dt == DataType.ARRAY:
@@ -903,7 +906,7 @@ class _GXCommon:
                 info.xml.appendEndTag(TranslatorTags.CONTENTS_DESCRIPTION)
                 info.xml.appendStartTag(TranslatorTags.ARRAY_CONTENTS, None, None, True)
                 if info.xml.outputType == TranslatorOutputType.STANDARD_XML:
-                    info.xml.append(buff.remainingHexString(False))
+                    info.xml.append(buff.remainingHexStringFalse)
                     info.xml.appendEndTag(TranslatorTags.ARRAY_CONTENTS, True)
                     info.xml.appendEndTag(info.xml.getDataType(DataType.COMPACT_ARRAY))
             cls.getCompactArrayItem(buff, dt, list_, len_)
@@ -934,7 +937,7 @@ class _GXCommon:
         value = None
         #  If there is not enough data available.
         if len(buff) - buff.position < 1:
-            info.complete = (False)
+            info.complete = False
             return None
         value = buff.getUInt8() & 0xFF
         if info.xml:
@@ -955,7 +958,7 @@ class _GXCommon:
         value = None
         #  If there is not enough data available.
         if len(buff) - buff.position < 2:
-            info.complete = (False)
+            info.complete = False
             return None
         value = int(buff.getInt16())
         if info.xml:
@@ -976,7 +979,7 @@ class _GXCommon:
         value = None
         #  If there is not enough data available.
         if len(buff) - buff.position < 1:
-            info.complete = (False)
+            info.complete = False
             return None
         value = int(buff.getInt8())
         if info.xml:
@@ -994,7 +997,7 @@ class _GXCommon:
     def getBcd(cls, buff, info):
         #  If there is not enough data available.
         if len(buff) - buff.position < 1:
-            info.complete = (False)
+            info.complete = False
             return None
         value = buff.getUInt8()
         if info.xml:
@@ -1018,7 +1021,7 @@ class _GXCommon:
             len_ = _GXCommon.getObjectCount(buff)
             #  If there is not enough data available.
             if len(buff) - buff.position < len_:
-                info.complete = (False)
+                info.complete = False
                 return None
         if len_ > 0:
             value = buff.getString(buff.position, len_)
@@ -1050,7 +1053,7 @@ class _GXCommon:
             len_ = _GXCommon.getObjectCount(buff)
             #  If there is not enough data available.
             if len(buff) - buff.position < len_:
-                info.complete = (False)
+                info.complete = False
                 return None
         tmp = bytearray(len_)
         buff.get(tmp)
@@ -1107,7 +1110,7 @@ class _GXCommon:
             len_ = _GXCommon.getObjectCount(buff)
             #  If there is not enough data available.
             if len(buff) - buff.position < len_:
-                info.complete = (False)
+                info.complete = False
                 return None
         if len_ > 0:
             value = buff.getString(buff.position, len_)
@@ -1154,7 +1157,7 @@ class _GXCommon:
     def getInt32(cls, buff, info):
         #  If there is not enough data available.
         if len(buff) - buff.position < 4:
-            info.complete = (False)
+            info.complete = False
             return None
         value = int(buff.getInt32())
         if info.xml:
@@ -1180,7 +1183,7 @@ class _GXCommon:
         byteCnt = int(t)
         #  If there is not enough data available.
         if len(buff) - buff.position < byteCnt:
-            info.complete = (False)
+            info.complete = False
             return None
         sb = ""
         while cnt > 0:
@@ -1203,7 +1206,7 @@ class _GXCommon:
     def getBoolean(cls, buff, info):
         #  If there is not enough data available.
         if len(buff) - buff.position < 1:
-            info.complete = (False)
+            info.complete = False
             return None
         value = bool(buff.getUInt8() != 0)
         if info.xml:
@@ -1628,6 +1631,8 @@ class _GXCommon:
             ret = DataType.OCTET_STRING
         #elif isinstance(value, (Enum,)):
         #    ret = DataType.ENUM
+        elif isinstance(value, bool):
+            ret = DataType.BOOLEAN
         elif isinstance(value, int):
             ret = DataType.INT32
         elif isinstance(value, GXTime):
@@ -1640,10 +1645,14 @@ class _GXCommon:
             ret = DataType.ARRAY
         elif isinstance(value, str):
             ret = DataType.STRING
-        elif isinstance(value, bool):
-            ret = DataType.BOOLEAN
         elif isinstance(value, float):
             ret = DataType.FLOAT32
+        elif isinstance(value, GXBitString):
+            ret = DataType.BITSTRING
+        elif isinstance(value, long):
+            ret = DataType.INT64
+        elif isinstance(value, unicode):
+            ret = DataType.STRING
         else:
             raise ValueError("Invalid value.")
         return ret
@@ -1769,11 +1778,11 @@ class _GXCommon:
 
     @classmethod
     def idisSystemTitleToString(cls, st):
-        sb = '\r'
-        sb += "IDIS system title:\r"
+        sb = '\n'
+        sb += "IDIS system title:\n"
         sb += "Manufacturer Code: "
         sb += cls.__getChar(st[0]) + cls.__getChar(st[1]) + cls.__getChar(st[2])
-        sb += "\rFunction type: "
+        sb += "\nFunction type: "
         ft = st[4] >> 4
         add = False
         if (ft & 0x1) != 0:
@@ -1794,15 +1803,15 @@ class _GXCommon:
         sn |= st[5] << 16
         sn |= st[6] << 8
         sn |= st[7]
-        sb += '\r'
+        sb += '\n'
         sb += "Serial number: "
-        sb += str(sn) + '\r'
+        sb += str(sn) + '\n'
         return sb
 
     @classmethod
     def dlmsSystemTitleToString(cls, st):
-        sb = '\r'
-        sb += "IDIS system title:\r"
+        sb = '\n'
+        sb += "IDIS system title:\n"
         sb += "Manufacturer Code: "
         sb += cls.__getChar(st[0]) + cls.__getChar(st[1]) + cls.__getChar(st[2])
         sb += "Serial number: "
@@ -1811,12 +1820,12 @@ class _GXCommon:
 
     @classmethod
     def uniSystemTitleToString(cls, st):
-        sb = '\r'
-        sb += "UNI/TS system title:\r"
+        sb = '\n'
+        sb += "UNI/TS system title:\n"
         sb += "Manufacturer: "
         m = st[0] << 8 | st[1]
         sb += cls.decryptManufacturer(m)
-        sb += "\rSerial number: "
+        sb += "\nSerial number: "
         sb += GXByteBuffer.toHex((st[7], st[6], st[5], st[4], st[3], st[2]), False)
         return sb
 
