@@ -37,7 +37,7 @@ from ..enums import ErrorCode
 from ..internal._GXCommon import _GXCommon
 from ..GXByteBuffer import GXByteBuffer
 from ..enums import DataType, ObjectType, Authentication, AccessMode, MethodAccessMode
-from .enums import AssociationStatus, ApplicationContextName
+from .enums import AssociationStatus
 from .GXDLMSObjectCollection import GXDLMSObjectCollection
 from .GXApplicationContextName import GXApplicationContextName
 from .GXxDLMSContextType import GXxDLMSContextType
@@ -306,7 +306,7 @@ class GXDLMSAssociationLogicalName(GXDLMSObject, IGXDLMSBase):
             for attributeAccess in buff[0]:
                 id_ = attributeAccess[0]
                 tmp = attributeAccess[1]
-                obj.setAccess(id_, AccessMode(tmp))
+                obj.setAccess(id_, tmp)
             for methodAccess in buff[1]:
                 id_ = methodAccess[0]
                 tmp = 0
@@ -317,7 +317,7 @@ class GXDLMSAssociationLogicalName(GXDLMSObject, IGXDLMSBase):
                         tmp = 0
                 else:
                     tmp = methodAccess[1]
-                obj.setMethodAccess(id_, MethodAccessMode(tmp))
+                obj.setMethodAccess(id_, tmp)
 
     def getUserList(self, settings):
         data = GXByteBuffer()
@@ -545,10 +545,10 @@ class GXDLMSAssociationLogicalName(GXDLMSObject, IGXDLMSBase):
                     self.authenticationMechanismName.jointIsoCtt = value[0]
                     self.authenticationMechanismName.country = value[1]
                     self.authenticationMechanismName.countryName = value[2]
-                    self.authenticationMechanismName.identifiedOrganization = value[4]
+                    self.authenticationMechanismName.identifiedOrganization = value[3]
                     self.authenticationMechanismName.dlmsUA = value[4]
                     self.authenticationMechanismName.authenticationMechanismName = value[5]
-                    self.authenticationMechanismName.mechanismId = Authentication(value[6])
+                    self.authenticationMechanismName.mechanismId = value[6]
 
     def setValue(self, settings, e):
         if e.index == 1:
@@ -600,8 +600,8 @@ class GXDLMSAssociationLogicalName(GXDLMSObject, IGXDLMSBase):
             e.error = ErrorCode.READ_WRITE_DENIED
 
     def load(self, reader):
-        self.clientSAP = int(reader.readElementContentAsInt("ClientSAP"))
-        self.serverSAP = int(reader.readElementContentAsInt("ServerSAP"))
+        self.clientSAP = reader.readElementContentAsInt("ClientSAP")
+        self.serverSAP = reader.readElementContentAsInt("ServerSAP")
         if reader.isStartElement("ApplicationContextName", True):
             self.applicationContextName.jointIsoCtt = reader.readElementContentAsInt("JointIsoCtt")
             self.applicationContextName.country = reader.readElementContentAsInt("Country")
@@ -609,7 +609,7 @@ class GXDLMSAssociationLogicalName(GXDLMSObject, IGXDLMSBase):
             self.applicationContextName.identifiedOrganization = reader.readElementContentAsInt("IdentifiedOrganization")
             self.applicationContextName.dlmsUA = reader.readElementContentAsInt("DlmsUA")
             self.applicationContextName.applicationContext = reader.readElementContentAsInt("ApplicationContext")
-            self.applicationContextName.contextId = ApplicationContextName(reader.readElementContentAsInt("ContextId"))
+            self.applicationContextName.contextId = reader.readElementContentAsInt("ContextId")
             reader.readEndElement("ApplicationContextName")
         if reader.isStartElement("XDLMSContextInfo", True):
             self.xDLMSContextInfo.conformance = reader.readElementContentAsInt("Conformance")
@@ -626,14 +626,14 @@ class GXDLMSAssociationLogicalName(GXDLMSObject, IGXDLMSBase):
             self.authenticationMechanismName.identifiedOrganization = reader.readElementContentAsInt("IdentifiedOrganization")
             self.authenticationMechanismName.dlmsUA = reader.readElementContentAsInt("DlmsUA")
             self.authenticationMechanismName.authenticationMechanismName = reader.readElementContentAsInt("AuthenticationMechanismName")
-            self.authenticationMechanismName.mechanismId = Authentication(reader.readElementContentAsInt("MechanismId"))
+            self.authenticationMechanismName.mechanismId = reader.readElementContentAsInt("MechanismId")
             reader.readEndElement("AuthenticationMechanismName")
         str_ = reader.readElementContentAsString("Secret")
         if str_ is None:
             self.secret = None
         else:
             self.secret = GXByteBuffer.hexToBytes(str_)
-        self.associationStatus = AssociationStatus(reader.readElementContentAsInt("AssociationStatus"))
+        self.associationStatus = reader.readElementContentAsInt("AssociationStatus")
         self.securitySetupReference = reader.readElementContentAsString("SecuritySetupReference")
 
     def save(self, writer):
@@ -666,7 +666,7 @@ class GXDLMSAssociationLogicalName(GXDLMSObject, IGXDLMSBase):
             writer.writeElementString("IdentifiedOrganization", self.authenticationMechanismName.identifiedOrganization)
             writer.writeElementString("DlmsUA", self.authenticationMechanismName.dlmsUA)
             writer.writeElementString("AuthenticationMechanismName", self.authenticationMechanismName.authenticationMechanismName)
-            writer.writeElementString("MechanismId", self.authenticationMechanismName.mechanismId)
+            writer.writeElementString("MechanismId", int(self.authenticationMechanismName.mechanismId))
             writer.writeEndElement()
         writer.writeElementString("Secret", GXByteBuffer.hex(self.secret))
         writer.writeElementString("AssociationStatus", self.associationStatus)
