@@ -37,6 +37,7 @@ from .GXDLMSObject import GXDLMSObject
 from ..enums import ObjectType
 from .GXXmlWriter import GXXmlWriter
 from .GXXmlReader import GXXmlReader
+from ..GXDLMSConverter import GXDLMSConverter
 
 class GXDLMSObjectCollection(list):
     """
@@ -58,6 +59,13 @@ class GXDLMSObjectCollection(list):
         super(GXDLMSObjectCollection, self).append(item)
         item.parent = self
 
+    def extend(self, items):
+        if not isinstance(items, GXDLMSObjectCollection):
+            raise TypeError('items is not of type GXDLMSObjectCollection')
+        for it in items:
+            super(GXDLMSObjectCollection, self).append(it)
+            it.parent = self
+
     def getObjects(self, type_):
         if isinstance(type_, (int, ObjectType)):
             type_ = [type_]
@@ -88,14 +96,6 @@ class GXDLMSObjectCollection(list):
         str_ += ']'
         return str_
 
-    @classmethod
-    def getName(cls, name):
-        name = "ObjectType." + name
-        for it in ObjectType:
-            if str(it).replace("_", "") == name:
-                return it
-        return None
-
     @classmethod
     def load(cls, file_):
         from .._GXObjectFactory import _GXObjectFactory
@@ -111,7 +111,7 @@ class GXDLMSObjectCollection(list):
                     reader.getNext()
                 elif target.find("GXDLMS") == 0:
                     try:
-                        type_ = cls.getName(target[6:].upper())
+                        type_ = GXDLMSConverter.valueOfObjectType(target[6:])
                     except Exception:
                         raise ValueError("Invalid object type: " + target + ".")
                     obj = _GXObjectFactory.createObject(type_)
@@ -130,23 +130,11 @@ class GXDLMSObjectCollection(list):
                 reader.read()
         return reader.objects
 
-    @classmethod
-    def getObjectName(cls, ot):
-        name = ot.name.lower()
-        tmp = name.split("_")
-        pos = 0
-        name = ""
-        while pos < len(tmp):
-            name += tmp[pos][0].upper()
-            name += tmp[pos][1:]
-            pos += 1
-        return name
-
     def save(self, name, settings=None):
         writer = GXXmlWriter()
         objects = ET.Element("objects")
         for it in self:
-            node = ET.SubElement(objects, "GXDLMS" + self.getObjectName(it.objectType))
+            node = ET.SubElement(objects, "GXDLMS" + GXDLMSConverter.objectTypeToString(it.objectType))
             if it.shortName != 0:
                 ET.SubElement(node, "SN").text = it.shortName
             ET.SubElement(node, "LN").text = it.logicalName
