@@ -31,13 +31,18 @@
 #  This code is licensed under the GNU General Public License v2.
 #  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
 # ---------------------------------------------------------------------------
-
 import xml.etree.cElementTree as ET
+try:
+    from enum import Enum, IntEnum
+except Exception:
+    pass
 from ..GXByteBuffer import GXByteBuffer
 from ..GXDateTime import GXDateTime
 from ..GXDLMSConverter import GXDLMSConverter
 from ..internal._GXCommon import _GXCommon
 from ..enums import DataType
+from ..GXArray import GXArray
+from ..GXStructure import GXStructure
 
 ###Python 2 requires this
 #pylint: disable=bad-option-value,old-style-class
@@ -90,33 +95,26 @@ class GXXmlWriter:
                 ET.SubElement(self.getTarget(), name).text = str(value)
             elif isinstance(value, (bytearray, bytes)):
                 ET.SubElement(self.getTarget(), name).text = GXByteBuffer.hex(value)
+            elif isinstance(value, (float)):
+                ET.SubElement(self.getTarget(), name).text = str(value).replace(",", ".")
 
     def writeArray(self, data):
-        if isinstance(data, []):
+        if isinstance(data, (list)):
             arr = data
             for tmp in arr:
                 if isinstance(tmp, bytearray):
-                    self.writeElementObject("Item", tmp, False)
-                elif isinstance(tmp, (object,)):
-                    self.writeStartElement("Item", "Type", str(DataType.ARRAY), True)
+                    self.writeElementObject("Item", tmp)
+                elif isinstance(tmp, (GXArray,)):
+                    self.writeStartElement("Item", "Type", str(int(DataType.ARRAY)), True)
+                    self.writeArray(tmp)
+                    self.writeEndElement()
+                elif isinstance(tmp, (GXStructure,)):
+                    self.writeStartElement("Item", "Type", str(int(DataType.STRUCTURE)), True)
                     self.writeArray(tmp)
                     self.writeEndElement()
                 else:
                     self.writeElementObject("Item", tmp)
 
-    def __writeElementObject(self, name, value, type_, uiType):
-        if type_ != DataType.NONE and isinstance(value, str):
-            if type_ == DataType.OCTET_STRING:
-                if uiType == DataType.STRING:
-                    self.writeElementObject(name, (str(value)).encode(), True)
-                elif uiType == DataType.OCTET_STRING:
-                    self.writeElementObject(name, GXByteBuffer.hexToBytes(str(value)), True)
-                return
-            if not isinstance(value, (GXDateTime)):
-                self.writeElementObject(name, GXDLMSConverter.changeType(value, type_), True)
-                return
-        self.writeElementObject(name, value, True)
-
     #
     # Write object value to file.
     #
@@ -124,29 +122,51 @@ class GXXmlWriter:
     # Object name.
     # @param value
     # Object value.
-    # @param skipDefaultValue
-    # Is default value serialized.
     #
     # pylint: disable=too-many-arguments
-    def writeElementObject(self, name, value, skipDefaultValue=True, type_=DataType.NONE, uiType=DataType.NONE):
-        if value or not skipDefaultValue:
-            if type_ == DataType.OCTET_STRING:
+    def writeElementObject(self, name, value, dt=DataType.NONE, uiType=DataType.NONE):
+        isEnum = False
+        try:
+            if isinstance(value, (Enum, IntEnum)):
+                isEnum = True
+        except (Exception):
+            pass
+        if isEnum:
+            raise ValueError("Datatype is enum.")
+
+        if value or not self.skipDefaults:
+            if value is None:
+                target = self.writeStartElement(name)
+                self.writeEndElement()
+                return
+            if dt == DataType.OCTET_STRING:
                 if uiType == DataType.STRING:
                     value = str(value)
                 elif uiType == DataType.OCTET_STRING:
-                    value = GXByteBuffer.hexToBytes(value)
-            elif not isinstance(value, GXDateTime):
-                value = GXDLMSConverter.changeType(value, type_)
+                    value = GXByteBuffer.hex(value)
+            elif dt != DataType.NONE and not isinstance(value, (float, GXDateTime)):
+                value = GXDLMSConverter.changeType(value, dt)
 
-            dt = _GXCommon.getDLMSDataType(value)
-            target = self.writeStartElement(name, "Type", str(dt), False)
+            if dt == DataType.NONE:
+                dt = _GXCommon.getDLMSDataType(value)
+
+            target = self.writeStartElement(name, "Type", str(int(dt)), False)
+            if uiType != DataType.NONE and uiType != dt and (uiType != DataType.STRING or dt == DataType.OCTET_STRING):
+                target.set("UIType", str(int(uiType)))
             if dt == DataType.ARRAY:
                 self.writeArray(value)
             else:
+                if isinstance(value, (float)):
+                    target.set("UIType", str(int(DataType.FLOAT64)))
                 if isinstance(value, GXDateTime):
                     target.text = value.toFormatString()
                 elif isinstance(value, (bytearray, bytes)):
                     target.text = GXByteBuffer.hex(value)
+                elif isinstance(value, bool):
+                    if value:
+                        target.text = "1"
+                    else:
+                        target.text = "0"
                 else:
                     target.text = str(value)
             self.writeEndElement()
